"""Constants for LLM models and parameters."""

from .anthropic_models import (
    ALL_CLAUDE_MODELS,
    ANTHROPIC_SERVICE_TIER_VALUES,
    CLAUDE_3_5_MODELS,
    CLAUDE_3_7_MODELS,
    CLAUDE_3_MODELS,
    CLAUDE_4_MODELS,
    DEFAULT_CHAT_MODEL,
    DEFAULT_FAST_MODEL,
    DEFAULT_REASONING_MODEL,
    DEFAULT_THINKING_CONFIG,
    DEFAULT_THINKING_MODEL,
    DEPRECATED_MODEL_REPLACEMENTS,
    HAIKU_MODELS,
    OPUS_MODELS,
    OUTPUT_EFFORT_VALUES,
    SONNET_MODELS,
    THINKING_DISABLED_CONFIG,
    THINKING_ENABLED_MODELS,
    create_thinking_config,
    get_model_family,
    get_model_replacement,
    get_model_size,
    is_claude_3_7,
    is_claude_4,
    is_deprecated,
    supports_thinking,
    validate_thinking_config,
)
from .anthropic_models import (
    DEFAULT_CHAT_MODEL as ANTHROPIC_DEFAULT_CHAT_MODEL,
)
from .anthropic_models import (
    DEFAULT_FAST_MODEL as ANTHROPIC_DEFAULT_FAST_MODEL,
)
from .anthropic_models import (
    DEFAULT_REASONING_MODEL as ANTHROPIC_DEFAULT_REASONING_MODEL,
)
from .anthropic_models import (
    DEFAULT_THINKING_MODEL as ANTHROPIC_DEFAULT_THINKING_MODEL,
)
from .anthropic_tool_constants import (
    create_anthropic_bash_tool,
    create_anthropic_code_execution_tool,
    create_anthropic_computer_use_tool,
    create_anthropic_function_tool,
    create_anthropic_memory_tool,
    create_anthropic_text_editor_tool,
    create_anthropic_tool_search_bm25_tool,
    create_anthropic_tool_search_regex_tool,
    create_anthropic_web_fetch_tool,
    create_anthropic_web_search_tool,
)
from .grammar_constants import (
    EXAMPLE_GRAMMARS,
    GRAMMAR_DEFAULT_TIMEOUT,
    GRAMMAR_LATENCY_MULTIPLIER,
    GRAMMAR_RESPONSE_FORMAT_TYPE,
    GRAMMAR_SYNTAX_TYPES,
    GrammarSyntax,
    GrammarSyntaxType,
    create_grammar_response_format,
    create_grammar_tool,
    validate_grammar_definition,
    validate_grammar_syntax,
)
from .openai_models import (
    AUDIO_MODELS,
    BALANCED,
    COMPUTER_USE_MODELS,
    GPT_4_1_MODELS,
    GPT_4O_MODELS,
    GPT_5_MODELS,
    GRAMMAR_SUPPORTED_MODELS,
    O_SERIES_MODELS,
    REALTIME_MODELS,
    REASONING_EFFORT_VALUES,
    SEARCH_MODELS,
    SERVICE_TIER_VALUES,
    SPEED_OPTIMIZED,
    THOROUGH,
    VERBOSITY_VALUES,
    requires_max_completion_tokens,
    supports_audio,
    supports_computer_use,
    supports_grammar,
    supports_realtime,
    supports_reasoning_effort,
    supports_verbosity,
)
from .openai_models import (
    DEFAULT_CHAT_MODEL as OPENAI_DEFAULT_CHAT_MODEL,
)
from .openai_models import (
    DEFAULT_FAST_MODEL as OPENAI_DEFAULT_FAST_MODEL,
)
from .openai_models import (
    DEFAULT_REASONING_MODEL as OPENAI_DEFAULT_REASONING_MODEL,
)
from .tool_constants import (
    create_apply_patch_tool,
    create_code_interpreter_tool,
    create_computer_tool,
    create_computer_use_preview_tool,
    create_file_search_tool,
    create_function_tool,
    create_image_generation_tool,
    create_local_shell_tool,
    create_mcp_tool,
    create_tool_search_tool,
    create_web_search_preview_tool,
    create_web_search_tool,
)

__all__ = [
    "ALL_CLAUDE_MODELS",
    "ANTHROPIC_DEFAULT_CHAT_MODEL",
    "ANTHROPIC_DEFAULT_FAST_MODEL",
    "ANTHROPIC_DEFAULT_REASONING_MODEL",
    "ANTHROPIC_DEFAULT_THINKING_MODEL",
    "ANTHROPIC_SERVICE_TIER_VALUES",
    "AUDIO_MODELS",
    "BALANCED",
    "CLAUDE_3_5_MODELS",
    "CLAUDE_3_7_MODELS",
    "CLAUDE_3_MODELS",
    # Anthropic constants
    "CLAUDE_4_MODELS",
    "COMPUTER_USE_MODELS",
    "DEFAULT_CHAT_MODEL",
    "DEFAULT_FAST_MODEL",
    "DEFAULT_REASONING_MODEL",
    "DEFAULT_THINKING_CONFIG",
    "DEFAULT_THINKING_MODEL",
    "DEPRECATED_MODEL_REPLACEMENTS",
    "EXAMPLE_GRAMMARS",
    "GPT_4O_MODELS",
    "GPT_4_1_MODELS",
    # OpenAI constants
    "GPT_5_MODELS",
    "GRAMMAR_DEFAULT_TIMEOUT",
    "GRAMMAR_LATENCY_MULTIPLIER",
    "GRAMMAR_RESPONSE_FORMAT_TYPE",
    "GRAMMAR_SUPPORTED_MODELS",
    # Grammar constants
    "GRAMMAR_SYNTAX_TYPES",
    "HAIKU_MODELS",
    "OPENAI_DEFAULT_CHAT_MODEL",
    "OPENAI_DEFAULT_FAST_MODEL",
    "OPENAI_DEFAULT_REASONING_MODEL",
    "OPUS_MODELS",
    "OUTPUT_EFFORT_VALUES",
    "O_SERIES_MODELS",
    "REALTIME_MODELS",
    "REASONING_EFFORT_VALUES",
    "SEARCH_MODELS",
    "SERVICE_TIER_VALUES",
    "SONNET_MODELS",
    "SPEED_OPTIMIZED",
    "THINKING_DISABLED_CONFIG",
    "THINKING_ENABLED_MODELS",
    "THOROUGH",
    "VERBOSITY_VALUES",
    "GrammarSyntax",
    "GrammarSyntaxType",
    "create_anthropic_bash_tool",
    "create_anthropic_code_execution_tool",
    "create_anthropic_computer_use_tool",
    "create_anthropic_function_tool",
    "create_anthropic_memory_tool",
    "create_anthropic_text_editor_tool",
    "create_anthropic_tool_search_bm25_tool",
    "create_anthropic_tool_search_regex_tool",
    "create_anthropic_web_fetch_tool",
    "create_anthropic_web_search_tool",
    "create_apply_patch_tool",
    "create_code_interpreter_tool",
    "create_computer_tool",
    "create_computer_use_preview_tool",
    "create_file_search_tool",
    "create_function_tool",
    "create_grammar_response_format",
    "create_grammar_tool",
    "create_image_generation_tool",
    "create_local_shell_tool",
    "create_mcp_tool",
    "create_thinking_config",
    "create_tool_search_tool",
    "create_web_search_preview_tool",
    "create_web_search_tool",
    "get_model_family",
    "get_model_replacement",
    "get_model_size",
    "is_claude_3_7",
    "is_claude_4",
    "is_deprecated",
    "requires_max_completion_tokens",
    "supports_audio",
    "supports_computer_use",
    "supports_grammar",
    "supports_realtime",
    "supports_reasoning_effort",
    "supports_thinking",
    "supports_verbosity",
    "validate_grammar_definition",
    "validate_grammar_syntax",
    "validate_thinking_config",
]
